/* 

 * $Header: /home/projects/mule/scm/mule/providers/email/src/java/org/mule/providers/email/MailMessageReceiver.java,v 1.2 2005/02/25 09:07:52 rossmason Exp $

 * $Revision: 1.2 $

 * $Date: 2005/02/25 09:07:52 $

 * ------------------------------------------------------------------------------------------------------

 * 

 * Copyright (c) Cubis Limited. All rights reserved.

 * http://www.cubis.co.uk 

 * 

 * The software in this package is published under the terms of the BSD

 * style license a copy of which has been included with this distribution in

 * the LICENSE.txt file. 

 *

 */


package org.mule.providers.email;


import org.mule.InitialisationException;
import org.mule.MuleManager;
import org.mule.impl.MuleMessage;
import org.mule.providers.PollingMessageReceiver;
import org.mule.umo.UMOComponent;
import org.mule.umo.UMOException;
import org.mule.umo.UMOMessage;
import org.mule.umo.endpoint.UMOEndpoint;
import org.mule.umo.provider.UMOConnector;
import org.mule.util.UUID;
import org.mule.util.Utility;

import javax.mail.Flags;
import javax.mail.Folder;
import javax.mail.Message;
import javax.mail.MessagingException;
import javax.mail.event.MessageCountEvent;
import javax.mail.event.MessageCountListener;
import javax.mail.internet.MimeMessage;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;


/**
 * <code>MailMessageReceiver</code> polls a pop3 mailbox for messages
 * removes the messages and routes them as events into Mule.
 *
 * @author <a href="mailto:ross.mason@cubis.co.uk">Ross Mason</a>
 * @version $Revision: 1.2 $
 */

public class MailMessageReceiver extends PollingMessageReceiver implements MessageCountListener
{
    private Folder folder = null;

    private String backupFolder = null;

    public MailMessageReceiver(UMOConnector connector,
                               UMOComponent component,
                               UMOEndpoint endpoint,
                               Folder folder,
                               Long checkFrequency, String backupFolder) throws InitialisationException
    {
        super(connector, component, endpoint, checkFrequency);

        setFolder(folder);
        if ("".equals(backupFolder))
        {
            this.backupFolder = MuleManager.getConfiguration().getWorkingDirectoy() + "/mail/" + folder.getName();
        }
        else
        {
            this.backupFolder = backupFolder;
        }
        if (backupFolder != null && !this.backupFolder.endsWith(File.separator))
        {
            this.backupFolder += File.separator;
        }
    }


    /* (non-Javadoc)
     * @see javax.mail.event.MessageCountListener#messagesAdded(javax.mail.event.MessageCountEvent)
     */
    public void messagesAdded(MessageCountEvent event)
    {
        Message messages[] = event.getMessages();
        Object msg = null;
        for (int i = 0; i < messages.length; i++)
        {
            try
            {
                if (!messages[i].getFlags().contains(Flags.Flag.DELETED))
                {
                    MimeMessage mimeMessage = new MimeMessage((MimeMessage)messages[i]);
                    storeMessage(mimeMessage);
                    UMOMessage message = new MuleMessage(connector.getMessageAdapter(mimeMessage));

                    // Mark as deleted
                    messages[i].setFlag(Flags.Flag.DELETED, true);
                    routeMessage(message, endpoint.isSynchronous());
                }
            }
            catch (Exception e)
            {
                handleException(msg, e);
            }
        }
    }

    protected UMOMessage handleUnacceptedFilter(UMOMessage message)
    {
        super.handleUnacceptedFilter(message);
        Message msg = (Message)message.getPayload();
        try
        {
            msg.setFlag(Flags.Flag.DELETED, endpoint.isDeleteUnacceptedMessages());
        } catch (MessagingException e)
        {
            logger.error("failled to set message deleted: " + e.getMessage(), e);
        }
        return null;
    }

    /* (non-Javadoc)
     * @see javax.mail.event.MessageCountListener#messagesRemoved(javax.mail.event.MessageCountEvent)
     */
    public void messagesRemoved(MessageCountEvent event)
    {
        if (logger.isDebugEnabled())
        {
            Message messages[] = event.getMessages();
            for (int i = 0; i < messages.length; i++)
            {
                try
                {
                    logger.debug("Message removed: " + messages[i].getSubject());
                }
                catch (MessagingException ignore)
                {
                }
            }
        }
    }

    /**
     * @return
     */
    public Folder getFolder()
    {
        return folder;
    }

    /**
     * @param folder
     */
    public synchronized void setFolder(Folder folder)
    {
        if(folder==null) throw new IllegalArgumentException("Mail folder cannot be null");
        this.folder = folder;
        synchronized (this.folder)
        {
            if (!this.folder.isOpen())
            {
                try
                {
                    //Depending on Server implementation it's not always necessary
                    //to open the folder to check it
                    //Opening folders can be exprensive!
                    //folder.open(Folder.READ_ONLY);
                    this.folder.open(Folder.READ_WRITE);
                }
                catch (MessagingException e)
                {
                    logger.warn("Failed to open folder: " + folder.getFullName(), e);
                }
//            try
//            {
//                folder.close(true);
//            }
//            catch (MessagingException me)
//            {
//                logger.warn("Failed to close folder: " + folder.getFullName(), me);
//            }
            }
        }
    }

    /**
     * Helper method for testing which stores a copy of the message locally as the POP3
     * <p/>
     * message will be deleted from the server
     *
     * @param msg the message to store
     * @throws IOException        If a failiure happens writing the message
     * @throws MessagingException If a failiure happens reading the message
     */
    private void storeMessage(Message msg) throws IOException, MessagingException
    {
        if (backupFolder != null)
        {
            String filename = msg.getFileName();
            if (filename == null)
            {
                filename = msg.getFrom()[0].toString() + "[" + new UUID().getUUID() + "]";
            }
            filename = Utility.prepareWinFilename(filename);
            filename = backupFolder + filename + ".msg";
            logger.debug("Writing message to: " + filename);
            File f = Utility.createFile(filename);
            FileWriter fw = new FileWriter(f);
            ByteArrayOutputStream baos = new ByteArrayOutputStream();
            msg.writeTo(baos);
            fw.write(new String(baos.toByteArray()));
            baos.close();
            fw.close();
        }
    }


    public void poll()
    {
        try
        {
            try
            {
                if(!folder.isOpen()) {
                    folder.open(Folder.READ_WRITE);
                }
            } catch (Exception e)
            {
                //ignore
            }

            int count = folder.getMessageCount();
            if (count > 0)
            {
                Message[] messages = folder.getMessages();
                MessageCountEvent event =
                        new MessageCountEvent(folder, MessageCountEvent.ADDED, true, messages);
                messagesAdded(event);
            }
            else if (count == -1)
            {
                throw new MessagingException("Cannot monitor folder: " + folder.getFullName() + " as folder is closed");
            }
            folder.close(true);				//close and expunge deleted messages
        }
        catch (MessagingException e)
        {
            handleException("Failed to read messages from folder: " + folder.getFullName(),
                    e);
        }
    }

    protected void doDispose() throws UMOException
    {
        super.doDispose();
        if(folder!=null ) folder.removeMessageCountListener(this);

        try
        {
             if(folder!=null) folder.close(false);
        } catch (MessagingException e)
        {
            logger.error("Failed to close pop3  inbox: " + e.getMessage());
        }
    }
}