/*
 * $Header: /home/projects/mule/scm/mule/mule/src/java/org/mule/routing/inbound/InboundMessageRouter.java,v 1.1.1.1 2005/02/15 02:27:56 topping Exp $
 * $Revision: 1.1.1.1 $
 * $Date: 2005/02/15 02:27:56 $
 * ------------------------------------------------------------------------------------------------------
 *
 * Copyright (c) Cubis Limited. All rights reserved.
 * http://www.cubis.co.uk
 *
 * The software in this package is published under the terms of the BSD
 * style license a copy of which has been included with this distribution in
 * the LICENSE.txt file.
 */
package org.mule.routing.inbound;

import EDU.oswego.cs.dl.util.concurrent.CopyOnWriteArrayList;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.mule.management.stats.RouterStatistics;
import org.mule.routing.AbstractRouterCollection;
import org.mule.umo.UMOEvent;
import org.mule.umo.UMOException;
import org.mule.umo.UMOMessage;
import org.mule.umo.endpoint.UMOEndpoint;
import org.mule.umo.routing.RoutingException;
import org.mule.umo.routing.UMOInboundMessageRouter;
import org.mule.umo.routing.UMOInboundRouter;

import java.util.Iterator;
import java.util.List;

/**
 * <code>InboundMessageRouter</code> is a collection of routers that will be invoked
 * when an event is received It is responsible for manageing a collection of routers and
 * also executing the routing logic. Each router must match against the current event for
 * the event to be routed.
 *
 * @author <a href="mailto:ross.mason@cubis.co.uk">Ross Mason </a>
 * @version $Revision: 1.1.1.1 $
 */

public class InboundMessageRouter extends AbstractRouterCollection implements
        UMOInboundMessageRouter
{
    /**
     * logger used by this class
     */
    protected static transient Log logger = LogFactory.getLog(InboundMessageRouter.class);

    private List endpoints = new CopyOnWriteArrayList();

    public InboundMessageRouter()
    {
        super(RouterStatistics.TYPE_INBOUND);
    }

    public UMOMessage route(UMOEvent event) throws RoutingException
    {
        if (endpoints.size() > 0 && routers.size() == 0)
        {
            addRouter(new InboundPassThroughRouter());
        }

        String componentName = event.getSession().getComponent().getDescriptor().getName();

        UMOEvent[] eventsToRoute = null;
        boolean noRoute = true;
        boolean match = false;
        UMOInboundRouter umoInboundRouter = null;

        for (Iterator iterator = getRouters().iterator(); iterator.hasNext();)
        {
            umoInboundRouter = (UMOInboundRouter) iterator.next();

            if (umoInboundRouter.isMatch(event))
            {
                match = true;
                eventsToRoute = umoInboundRouter.process(event);
                noRoute = (eventsToRoute == null);
                if(!matchAll) break;
            }
        }

        if (noRoute)
        {
            //Update stats
            if (getStatistics().isEnabled())
            {
                getStatistics().incrementNoRoutedMessage();
            }
            if (!match)
            {
                if (getCatchAllStrategy() != null)
                {

                    logger.debug("Message did not match any routers on: " + componentName
                            + " invoking catch all strategy");
                    getStatistics().incrementCaughtMessage();
                    return getCatchAllStrategy().catchMessage(event.getMessage(), event.getSession(), event.isSynchronous());

                } else
                {
                    logger.warn("Message did not match any routers on: "
                            + componentName
                            + " and there is no catch all strategy configured on this router.  Disposing message.");
                    if (logger.isDebugEnabled())
                    {
                        try
                        {
                            logger.warn("Message fragment is: " + event.getMessageAsString().substring(0, 100) + "...");
                        } catch (UMOException e)
                        {

                        }
                    }
                }
            }
        } else
        {
            try
            {
                if (event.isSynchronous())
                {
                    UMOMessage messageResult = null;
                    for (int i = 0; i < eventsToRoute.length; i++)
                    {
                        messageResult = send(eventsToRoute[i]);
                        //Update stats
                        if (getStatistics().isEnabled())
                        {
                            getStatistics().incrementRoutedMessage(eventsToRoute[i].getEndpoint());
                        }
                    }
                    return messageResult;
                } else
                {
                    synchronized (eventsToRoute)
                    {
                        for (int i = 0; i < eventsToRoute.length; i++)
                        {
                            dispatch(eventsToRoute[i]);
                            if (getStatistics().isEnabled())
                            {
                                getStatistics().incrementRoutedMessage(eventsToRoute[i].getEndpoint());
                            }
                        }
                    }
                    return null;
                }
            } catch (UMOException e)
            {
                throw new RoutingException("Failed to send/dispatch event: "
                        + e.getMessage(), e);
            }
        }
        return (eventsToRoute != null && eventsToRoute.length > 0 ? eventsToRoute[eventsToRoute.length - 1].getMessage() : null);

    }

    public void dispatch(UMOEvent event) throws UMOException
    {
        event.getSession().dispatchEvent(event);
    }

    public UMOMessage send(UMOEvent event) throws UMOException
    {

        return event.getSession().sendEvent(event);
    }

    public void addRouter(UMOInboundRouter router)
    {
        routers.add(router);
    }

    public UMOInboundRouter removeRouter(UMOInboundRouter router)
    {
        if (routers.remove(router))
        {
            return router;
        } else
        {
            return null;
        }
    }

    public void addEndpoint(UMOEndpoint endpoint)
    {
        if (endpoint != null)
        {
            endpoint.setType(UMOEndpoint.ENDPOINT_TYPE_RECEIVER);
            endpoints.add(endpoint);
        } else
        {
            throw new NullPointerException("Endpoint cannot be null");
        }
    }

    public boolean removeEndpoint(UMOEndpoint endpoint)
    {
        return endpoints.remove(endpoint);
    }

    public List getEndpoints()
    {
        return endpoints;
    }

    public void setEndpoints(List endpoints)
    {
        this.endpoints = endpoints;
    }

    /**
     * @param name the Endpoint identifier
     * @return the Endpoint or null if the endpointUri is not registered
     * @see org.mule.umo.routing.UMOInboundMessageRouter
     */
    public UMOEndpoint getEndpoint(String name)
    {
        UMOEndpoint endpointDescriptor;
        for (Iterator iterator = endpoints.iterator(); iterator.hasNext();)
        {
            endpointDescriptor = (UMOEndpoint) iterator.next();
            if (endpointDescriptor.getName().equals(name))
            {
                return endpointDescriptor;
            }
        }
        return null;
    }
}