/*
 * $Header: /home/projects/mule/scm/mule/mule/src/java/org/mule/util/monitor/ExpiryMonitor.java,v 1.2 2005/02/21 23:10:08 rossmason Exp $
 * $Revision: 1.2 $
 * $Date: 2005/02/21 23:10:08 $
 * ------------------------------------------------------------------------------------------------------
 *
 * Copyright (c) Cubis Limited. All rights reserved.
 * http://www.cubis.co.uk
 *
 * The software in this package is published under the terms of the BSD
 * style license a copy of which has been included with this distribution in
 * the LICENSE.txt file.
 */
package org.mule.util.monitor;

import EDU.oswego.cs.dl.util.concurrent.ConcurrentHashMap;
import org.mule.umo.lifecycle.Disposable;
import org.apache.commons.logging.LogFactory;
import org.apache.commons.logging.Log;

import java.util.Iterator;
import java.util.Map;
import java.util.Timer;
import java.util.TimerTask;

/**
 * <code>ExpiryMonitor</code> can monitor objects beased on an expiry time and
 * can invoke a callback method once the object time has expired.  If the object
 * does expire it is removed from this monitor
 *
 * @author <a href="mailto:ross.mason@cubis.co.uk">Ross Mason</a>
 * @version $Revision: 1.2 $
 */

public class ExpiryMonitor extends TimerTask implements Disposable
{
    /**
     * logger used by this class
     */
    protected static transient Log logger = LogFactory.getLog(ExpiryMonitor.class);

    private Timer timer;
    private Map monitors;

    public ExpiryMonitor()
    {
        this(1000);
    }

    public ExpiryMonitor(long monitorFrequency)
    {

        timer = new Timer(true);
        timer.schedule(this, monitorFrequency, monitorFrequency);
        monitors = new ConcurrentHashMap();
    }


    /**
     * Adds an expirable object to monitor.  If the Object is
     * already being monitored it will be reset and the
     * millisecond timeout will be ignored
     * @param milliseconds
     * @param expirable
     */
    public void addExpirable(long milliseconds, Expirable expirable)
    {
        if(isRegistered(expirable)) {
            resetExpirable(expirable);
        } else {
            if(logger.isDebugEnabled()) logger.debug("Adding new expirable: " + expirable);
            monitors.put(expirable, new ExpriableHolder(milliseconds, expirable));
        }
    }

    public boolean isRegistered( Expirable expirable)
    {
        return (monitors.get(expirable)!=null);
    }

    public void removeExpirable(Expirable expirable)
    {
        if(logger.isDebugEnabled()) logger.debug("Removing expirable: " + expirable);
        monitors.remove(expirable);
    }

    public void resetExpirable(Expirable expirable)
    {
        ExpriableHolder eh = (ExpriableHolder)monitors.get(expirable);
        if(eh!=null) {
            eh.reset();
            if(logger.isDebugEnabled()) logger.debug("Reset expirable: " + expirable);
        }
    }

    /**
     * The action to be performed by this timer task.
     */
    public void run()
    {
        ExpriableHolder holder;
        for (Iterator iterator = monitors.values().iterator(); iterator.hasNext();)
        {
            holder = (ExpriableHolder)iterator.next();
            if(holder.isExpired()) {
                removeExpirable(holder.getExpirable());
                holder.getExpirable().expired();
            }
        }
    }

    public void dispose()
    {
        logger.info("disposing monitor");
        timer.cancel();
        ExpriableHolder holder;
        for (Iterator iterator = monitors.values().iterator(); iterator.hasNext();)
        {
            holder = (ExpriableHolder)iterator.next();
            removeExpirable(holder.getExpirable());
            try
            {
                holder.getExpirable().expired();
            } catch (Exception e)
            {
                logger.debug(e.getMessage());
            }
        }
    }

    private class ExpriableHolder {

        private long milliseconds;
        private Expirable expirable;
        private long created;

        public ExpriableHolder(long milliseconds, Expirable expirable)
        {
            this.milliseconds = milliseconds;
            this.expirable = expirable;
            created = System.currentTimeMillis();
        }

        public long getMilliseconds()
        {
            return milliseconds;
        }

        public Expirable getExpirable()
        {
            return expirable;
        }

        public boolean isExpired()
        {
            return  (System.currentTimeMillis() - milliseconds) > created;
        }

        public void reset() {
            created = System.currentTimeMillis();
        }
    }
}
